@@ -264,7 +264,6 @@ delete_values (hive_h *h, hive_node_h node)
     int is_inline;
     len = le32toh (vk->data_len);
     is_inline = !!(len & 0x80000000); /* top bit indicates is inline */
-    len &= 0x7fffffff;
 
     if (!is_inline) {           /* non-inline, so remove data block */
       size_t data_offset = le32toh (vk->data_offset);
@@ -340,7 +339,7 @@ insert_lf_record (hive_h *h, size_t old_offs, size_t posn,
   /* Work around C stupidity.
    * http://www.redhat.com/archives/libguestfs/2010-February/msg00056.html
    */
-  int test = BLOCK_ID_EQ (h, old_offs, "lf") || BLOCK_ID_EQ (h, old_offs, "lh");
+  int test = block_id_eq (h, old_offs, "lf") || block_id_eq (h, old_offs, "lh");
   assert (test);
 
   struct ntreg_lf_record *old_lf =
@@ -399,7 +398,7 @@ insert_li_record (hive_h *h, size_t old_offs, size_t posn,
                   const char *name, hive_node_h node)
 {
   assert (IS_VALID_BLOCK (h, old_offs));
-  assert (BLOCK_ID_EQ (h, old_offs, "li"));
+  assert (block_id_eq (h, old_offs, "li"));
 
   struct ntreg_ri_record *old_li =
     (struct ntreg_ri_record *) ((char *) h->addr + old_offs);
@@ -452,7 +451,7 @@ static int
 compare_name_with_nk_name (hive_h *h, const char *name, hive_node_h nk_offs)
 {
   assert (IS_VALID_BLOCK (h, nk_offs));
-  assert (BLOCK_ID_EQ (h, nk_offs, "nk"));
+  assert (block_id_eq (h, nk_offs, "nk"));
 
   /* Name in nk is not necessarily nul-terminated. */
   char *nname = hivex_node_name (h, nk_offs);
@@ -493,7 +492,7 @@ insert_subkey (hive_h *h, const char *name,
    * the end.
    */
   for (i = 0; blocks[i] != 0; ++i) {
-    if (BLOCK_ID_EQ (h, blocks[i], "lf") || BLOCK_ID_EQ (h, blocks[i], "lh")) {
+    if (block_id_eq (h, blocks[i], "lf") || block_id_eq (h, blocks[i], "lh")) {
       old_offs = blocks[i];
       old_lf = (struct ntreg_lf_record *) ((char *) h->addr + old_offs);
       old_li = NULL;
@@ -504,7 +503,7 @@ insert_subkey (hive_h *h, const char *name,
           goto insert_it;
       }
     }
-    else if (BLOCK_ID_EQ (h, blocks[i], "li")) {
+    else if (block_id_eq (h, blocks[i], "li")) {
       old_offs = blocks[i];
       old_lf = NULL;
       old_li = (struct ntreg_ri_record *) ((char *) h->addr + old_offs);
@@ -565,7 +564,7 @@ insert_subkey (hive_h *h, const char *name,
   }
   else {
     for (i = 0; blocks[i] != 0; ++i) {
-      if (BLOCK_ID_EQ (h, blocks[i], "ri")) {
+      if (block_id_eq (h, blocks[i], "ri")) {
         struct ntreg_ri_record *ri =
           (struct ntreg_ri_record *) ((char *) h->addr + blocks[i]);
         for (j = 0; j < le16toh (ri->nr_offsets); ++j)
@@ -593,7 +592,7 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
 {
   CHECK_WRITABLE (0);
 
-  if (!IS_VALID_BLOCK (h, parent) || !BLOCK_ID_EQ (h, parent, "nk")) {
+  if (!IS_VALID_BLOCK (h, parent) || !block_id_eq (h, parent, "nk")) {
     SET_ERRNO (EINVAL, "invalid block or not an 'nk' block");
     return 0;
   }
@@ -610,7 +609,8 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
 
   size_t recoded_name_len;
   int use_utf16 = 0;
-  char* recoded_name = _hivex_encode_string (name, &recoded_name_len, &use_utf16);
+  char *recoded_name =
+    _hivex_encode_string (name, &recoded_name_len, &use_utf16);
   if (recoded_name == NULL) {
     SET_ERRNO (EINVAL, "malformed name");
     return 0;
@@ -620,8 +620,10 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
   static const char nk_id[2] = { 'n', 'k' };
   size_t seg_len = sizeof (struct ntreg_nk_record) + recoded_name_len;
   hive_node_h nkoffset = allocate_block (h, seg_len, nk_id);
-  if (nkoffset == 0)
+  if (nkoffset == 0) {
+    free (recoded_name);
     return 0;
+  }
 
   DEBUG (2, "allocated new nk-record for child at 0x%zx", nkoffset);
 
@@ -646,7 +648,7 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
   size_t parent_sk_offset = le32toh (parent_nk->sk);
   parent_sk_offset += 0x1000;
   if (!IS_VALID_BLOCK (h, parent_sk_offset) ||
-      !BLOCK_ID_EQ (h, parent_sk_offset, "sk")) {
+      !block_id_eq (h, parent_sk_offset, "sk")) {
     SET_ERRNO (EFAULT,
                "parent sk is not a valid block (%zu)", parent_sk_offset);
     return 0;
@@ -704,7 +706,7 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
     /* Recalculate pointers that could have been invalidated by
      * previous call to allocate_block (via new_lh_record).
      */
-    nk = (struct ntreg_nk_record *) ((char *) h->addr + nkoffset);
+    /* nk could be invalid here */
     parent_nk = (struct ntreg_nk_record *) ((char *) h->addr + parent);
 
     DEBUG (2, "no keys, allocated new lh-record at 0x%zx", lh_offs);
@@ -720,7 +722,7 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
     /* Recalculate pointers that could have been invalidated by
      * previous call to allocate_block (via new_lh_record).
      */
-    nk = (struct ntreg_nk_record *) ((char *) h->addr + nkoffset);
+    /* nk could be invalid here */
     parent_nk = (struct ntreg_nk_record *) ((char *) h->addr + parent);
   }
 
@@ -744,7 +746,7 @@ hivex_node_add_child (hive_h *h, hive_node_h parent, const char *name)
 static int
 delete_sk (hive_h *h, size_t sk_offset)
 {
-  if (!IS_VALID_BLOCK (h, sk_offset) || !BLOCK_ID_EQ (h, sk_offset, "sk")) {
+  if (!IS_VALID_BLOCK (h, sk_offset) || !block_id_eq (h, sk_offset, "sk")) {
     SET_ERRNO (EFAULT, "not an sk record: 0x%zx", sk_offset);
     return -1;
   }
@@ -849,7 +851,7 @@ hivex_node_delete_child (hive_h *h, hive_node_h node)
 {
   CHECK_WRITABLE (-1);
 
-  if (!IS_VALID_BLOCK (h, node) || !BLOCK_ID_EQ (h, node, "nk")) {
+  if (!IS_VALID_BLOCK (h, node) || !block_id_eq (h, node, "nk")) {
     SET_ERRNO (EINVAL, "invalid block or not an 'nk' block");
     return -1;
   }
@@ -922,7 +924,7 @@ hivex_node_set_values (hive_h *h, hive_node_h node,
 {
   CHECK_WRITABLE (-1);
 
-  if (!IS_VALID_BLOCK (h, node) || !BLOCK_ID_EQ (h, node, "nk")) {
+  if (!IS_VALID_BLOCK (h, node) || !block_id_eq (h, node, "nk")) {
     SET_ERRNO (EINVAL, "invalid block or not an 'nk' block");
     return -1;
   }
@@ -947,8 +949,6 @@ hivex_node_set_values (hive_h *h, hive_node_h node,
   nk->nr_values = htole32 (nr_values);
   nk->vallist = htole32 (vallist_offs - 0x1000);
 
-  struct ntreg_value_list *vallist =
-    (struct ntreg_value_list *) ((char *) h->addr + vallist_offs);
 
   size_t i;
   for (i = 0; i < nr_values; ++i) {
@@ -967,7 +967,8 @@ hivex_node_set_values (hive_h *h, hive_node_h node,
      * previous call to allocate_block.
      */
     nk = (struct ntreg_nk_record *) ((char *) h->addr + node);
-    vallist = (struct ntreg_value_list *) ((char *) h->addr + vallist_offs);
+    struct ntreg_value_list *vallist =
+      (struct ntreg_value_list *) ((char *) h->addr + vallist_offs);
 
     vallist->offset[i] = htole32 (vk_offs - 0x1000);
 
@@ -997,7 +998,7 @@ hivex_node_set_values (hive_h *h, hive_node_h node,
        * previous call to allocate_block.
        */
       nk = (struct ntreg_nk_record *) ((char *) h->addr + node);
-      vallist = (struct ntreg_value_list *) ((char *) h->addr + vallist_offs);
+      /* vallist could be invalid here */
       vk = (struct ntreg_vk_record *) ((char *) h->addr + vk_offs);
 
       memcpy ((char *) h->addr + offs + 4, values[i].value, values[i].len);
